@@ -1,61 +1,22 @@
 #' Navigation containers
 #'
-#' Render a collection of [nav()] items into a container.
+#' Render a collection of [nav_panel()] items into a container.
 #'
-#' @param ... a collection of [nav()] items.
-#' @param id a character string used for dynamically updating the container (see [nav_select()]).
-#' @param selected a character string matching the `value` of a particular [nav()] item to selected by default.
+#' @includeRmd man/fragments/ex-navset_tab.Rmd
+#'
+#' @param ... a collection of [nav_panel()] items.
+#' @param id a character string used for dynamically updating the container (see
+#'    [nav_select()]).
+#' @param selected a character string matching the `value` of a particular
+#'   [nav_panel()] item to selected by default.
 #' @param header UI element(s) ([tags]) to display _above_ the nav content.
 #' @param footer UI element(s) ([tags]) to display _below_ the nav content.
+#'
+#' @seealso [nav_panel()], [nav_select()].
+#' @rdname navset
+#' @name navset
 #' @export
-#' @seealso [nav()], [nav_select()].
-#' @rdname navs
-#' @examples
-#'
-#' library(shiny)
-#'
-#' nav_items <- function(prefix) {
-#'   list(
-#'     nav("a", paste(prefix, ": tab a content")),
-#'     nav("b", paste(prefix, ": tab b content")),
-#'     nav_item(
-#'       tags$a(icon("github"), "Shiny", href = "https://github.com/rstudio/shiny", target = "_blank")
-#'     ),
-#'     nav_spacer(),
-#'     nav_menu(
-#'       "Other links", align = "right",
-#'       nav("c", paste(prefix, ": tab c content")),
-#'       nav_item(
-#'         tags$a(icon("r-project"), "RStudio", href = "https://rstudio.com", target = "_blank")
-#'       )
-#'     )
-#'   )
-#' }
-#'
-#' if (interactive()) {
-#'   shinyApp(
-#'     page_navbar(
-#'       title = "page_navbar()",
-#'       bg = "#0062cc",
-#'       !!!nav_items("page_navbar()"),
-#'       footer = div(
-#'         style = "width:80%; margin: 0 auto",
-#'         h4("navs_tab()"),
-#'         navs_tab(!!!nav_items("navs_tab()")),
-#'         h4("navs_pill()"),
-#'         navs_pill(!!!nav_items("navs_pill()")),
-#'         h4("navs_tab_card()"),
-#'         navs_tab_card(!!!nav_items("navs_tab_card()")),
-#'         h4("navs_pill_card()"),
-#'         navs_pill_card(!!!nav_items("navs_pill_card()")),
-#'         h4("navs_pill_list()"),
-#'         navs_pill_list(!!!nav_items("navs_pill_list()"))
-#'       )
-#'     ),
-#'     function(...) { }
-#'   )
-#' }
-navs_tab <- function(..., id = NULL, selected = NULL,
+navset_tab <- function(..., id = NULL, selected = NULL,
                      header = NULL, footer = NULL) {
   tabs <- tabsetPanel_(
     ..., type = "tabs", id = id, selected = selected,
@@ -65,8 +26,8 @@ navs_tab <- function(..., id = NULL, selected = NULL,
 }
 
 #' @export
-#' @rdname navs
-navs_pill <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_pill <- function(..., id = NULL, selected = NULL,
                       header = NULL, footer = NULL) {
   pills <- tabsetPanel_(
     ..., type = "pills", id = id, selected = selected,
@@ -77,8 +38,8 @@ navs_pill <- function(..., id = NULL, selected = NULL,
 
 #' @export
 #' @inheritParams shiny::navlistPanel
-#' @rdname navs
-navs_pill_list <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_pill_list <- function(..., id = NULL, selected = NULL,
                            header = NULL, footer = NULL,
                            well = TRUE, fluid = TRUE,
                            widths = c(4, 8)) {
@@ -92,8 +53,8 @@ navs_pill_list <- function(..., id = NULL, selected = NULL,
 }
 
 #' @export
-#' @rdname navs
-navs_hidden <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_hidden <- function(..., id = NULL, selected = NULL,
                         header = NULL, footer = NULL) {
   hidden <- tabsetPanel_(
     ..., type = "hidden", id = id, selected = selected,
@@ -102,19 +63,54 @@ navs_hidden <- function(..., id = NULL, selected = NULL,
   as_fragment(hidden)
 }
 
-
 #' @inheritParams shiny::navbarPage
+#' @inheritParams page_fillable
+#' @param sidebar A [sidebar()] component to display on every [nav_panel()]
+#'   page.
+#' @param fillable Whether or not to allow `fill` items to grow/shrink to fit
+#'   the browser window. If `TRUE`, all [nav_panel()] pages are `fillable`. A
+#'   character vector, matching the `value` of [nav_panel()]s to be filled, may
+#'   also be provided. Note that, if a `sidebar` is provided, `fillable` makes
+#'   the main content portion fillable.
 #' @param bg a CSS color to use for the navbar's background color.
 #' @param inverse Either `TRUE` for a light text color or `FALSE` for a dark
 #'   text color. If `"auto"` (the default), the best contrast to `bg` is chosen.
 #' @export
-#' @rdname navs
-navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
+#' @rdname navset
+navset_bar <- function(..., title = NULL, id = NULL, selected = NULL,
+                     sidebar = NULL, fillable = TRUE,
+                     gap = NULL, padding = NULL,
                      # TODO: add sticky-top as well?
                      position = c("static-top", "fixed-top", "fixed-bottom"),
                      header = NULL, footer = NULL,
                      bg = NULL, inverse = "auto",
                      collapsible = TRUE, fluid = TRUE) {
+  navs_bar_(
+    ..., title = title, id = id, selected = selected,
+    sidebar = sidebar, fillable = fillable,
+    gap = gap, padding = padding,
+    position = position,
+    header = header, footer = footer,
+    bg = bg, inverse = inverse,
+    collapsible = collapsible, fluid = fluid,
+    # theme is only used to determine whether legacy style markup should be used
+    # (and, at least at the moment, we don't need legacy markup for this exported function)
+    theme = bs_theme()
+  )
+}
+
+# This internal version of navs_bar() exists so both it and page_navbar()
+# (and thus shiny::navbarPage()) can use it. And in the page_navbar() case,
+# we can use addition theme information as an indication of whether we need
+# to handle backwards compatibility
+navs_bar_ <- function(..., title = NULL, id = NULL, selected = NULL,
+                      sidebar = NULL, fillable = TRUE,
+                      gap = NULL, padding = NULL,
+                      position = c("static-top", "fixed-top", "fixed-bottom"),
+                      header = NULL, footer = NULL,
+                      bg = NULL, inverse = "auto",
+                      collapsible = TRUE, fluid = TRUE,
+                      theme = NULL) {
 
   if (identical(inverse, "auto")) {
     inverse <- TRUE
@@ -127,9 +123,12 @@ navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
 
   navbar <- navbarPage_(
     title = title, ..., id = id, selected = selected,
+    sidebar = sidebar, fillable = fillable,
+    gap = gap, padding = padding,
     position = match.arg(position),
     header = header, footer = footer, collapsible = collapsible,
-    inverse = inverse, fluid = fluid
+    inverse = inverse, fluid = fluid,
+    theme = theme
   )
 
   if (!is.null(bg)) {
@@ -142,8 +141,6 @@ navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
   as_fragment(navbar, page = page)
 }
 
-
-
 # -----------------------------------------------------------------------
 # 'Internal' tabset logic that was pulled directly from shiny/R/bootstrap.R
 #  (with minor modifications)
@@ -153,15 +150,17 @@ navbarPage_ <- function(title,
                        ...,
                        id = NULL,
                        selected = NULL,
+                       sidebar = NULL,
+                       fillable = TRUE,
+                       gap = NULL,
+                       padding = NULL,
                        position = c("static-top", "fixed-top", "fixed-bottom"),
                        header = NULL,
                        footer = NULL,
                        inverse = FALSE,
                        collapsible = FALSE,
                        fluid = TRUE,
-                       theme = NULL,
-                       windowTitle = title,
-                       lang = NULL) {
+                       theme = NULL) {
 
   # alias title so we can avoid conflicts w/ title in withTags
   pageTitle <- title
@@ -197,7 +196,7 @@ navbarPage_ <- function(title,
           # data-bs-* is for BS5+
           `data-bs-toggle` = "collapse",
           `data-bs-target` = paste0("#", navId),
-          span(class="sr-only", "Toggle navigation"),
+          span(class="sr-only visually-hidden", "Toggle navigation"),
           span(class = "icon-bar"),
           span(class = "icon-bar"),
           span(class = "icon-bar")
@@ -230,13 +229,54 @@ navbarPage_ <- function(title,
       allTags()
   }
 
-  # build the main tab content div
-  contentDiv <- div(class = containerClass)
-  if (!is.null(header))
-    contentDiv <- tagAppendChild(contentDiv, div(class = "row", header))
-  contentDiv <- tagAppendChild(contentDiv, tabset$content)
-  if (!is.null(footer))
-    contentDiv <- tagAppendChild(contentDiv, div(class = "row", footer))
+  # If fillable is truthy, give the relevant .tab-content > .tab-pane containers
+  # the potential to fill
+  tabset$content <- makeTabsFillable(tabset$content, fillable, navbar = TRUE, gap = gap, padding = padding)
+
+  # For backwards compatibility reasons, wrap header & footer in a .row
+  # container if we're not using BS5+. I'm not entirely sure what the motivation
+  # was for it in the 1st place, but now that, with BS5, .row adds
+  # `display:flex` and makes children `width:100%`, which is surprising and
+  # confusing from a user perspective
+  isLegacy <- as.numeric(theme_version(theme) %||% 3) < 5
+  if (!is.null(header) && isLegacy) {
+    header <- div(class = "row", header)
+  }
+  if (!is.null(footer) && isLegacy) {
+    footer <- div(class = "row", footer)
+  }
+
+  contents <- dropNulls(list(header, tabset$content, footer))
+
+  if (is.null(sidebar)) {
+
+    contentDiv <- div(class = containerClass, !!!contents)
+
+    # If fillable is truthy, the .container also needs to be fillable
+    if (!isFALSE(fillable)) {
+      contentDiv <- bindFillRole(contentDiv, container = TRUE, item = TRUE)
+    }
+
+  } else {
+
+    contentDiv <- div(
+      # In the fluid case, the sidebar layout should be flush (i.e.,
+      # the .container-fluid class adds padding that we don't want)
+      class = if (!fluid) "container",
+      layout_sidebar(
+        sidebar = sidebar,
+        fillable = !isFALSE(fillable),
+        border_radius = FALSE,
+        border = !fluid,
+        contents
+      )
+    )
+
+    # Always have the sidebar layout fill its parent (in this case
+    # fillable controls whether the _main_ content portion is fillable)
+    contentDiv <- bindFillRole(contentDiv, container = TRUE, item = TRUE)
+
+  }
 
   # *Don't* wrap in bootstrapPage() (shiny::navbarPage()) does that part
   tagList(
@@ -475,7 +515,7 @@ buildTabItem <- function(index, tabsetId, foundSelected, tabs = NULL,
     # tabPanelMenu item: build the child tabset
     ulClass <- "dropdown-menu"
     if (identical(divTag$align, "right")) {
-      ulClass <- paste(ulClass, "dropdown-menu-right")
+      ulClass <- paste(ulClass, "dropdown-menu-right dropdown-menu-end")
     }
     tabset <- buildTabset(
       !!!divTag$tabs, ulClass = ulClass,
@@ -497,7 +537,7 @@ buildTabItem <- function(index, tabsetId, foundSelected, tabs = NULL,
 
   # The behavior is undefined at this point, so construct a condition message
   msg <- paste0(
-    "Navigation containers expect a collection of `bslib::nav()`/`shiny::tabPanel()`s ",
+    "Navigation containers expect a collection of `bslib::nav_panel()`/`shiny::tabPanel()`s ",
     "and/or `bslib::nav_menu()`/`shiny::navbarMenu()`s. ",
     "Consider using `header` or `footer` if you wish to place content above ",
     "(or below) every panel's contents."
@@ -584,7 +624,6 @@ buildDropdown <- function(divTag, tabset) {
 
   dropdown <- tags$li(
     class = "dropdown",
-    class = if (active) "active",
     tags$a(
       href = "#",
       class = "dropdown-toggle",
@@ -603,9 +642,10 @@ buildDropdown <- function(divTag, tabset) {
           addClass("nav-item")$
           find(".dropdown-toggle")$
           addClass("nav-link")$
+          addClass(if (active) "active")$
           allTags()
       } else {
-        x
+        tagAppendAttributes(x, class = if (active) "active")
       }
     }
   )
