@@ -15,7 +15,7 @@ extract_first_group <- function(x, pattern) {
 }
 
 download_and_copy_fonts <-  function(theme) {
-  theme_scss <- readLines(file.path(theme, "_bootswatch.scss"), warn = FALSE)
+  theme_scss <- readLines(file.path(theme, "_bootswatch.scss"))
   web_font_path <- grep("$web-font-path:", theme_scss, value = TRUE, fixed = TRUE)
   if (!length(web_font_path)) {
     return()
@@ -31,14 +31,32 @@ download_and_copy_fonts <-  function(theme) {
   urls <- sass:::extract_group(css, "url\\(([^)]+)")
   basenames <- basename(urls)
   Map(function(url, nm) {
-    sass:::download_file(url, file.path(fonts_home, nm))
+    target <- file.path(fonts_home, nm)
+    # The basename can sometimes be very long, and R CMD check
+    # will complain if the target file is over 100 characters long,
+    # so shorten it if necessary
+    if (nchar(target) > 100) {
+      nm <- paste0(
+        rlang::hash(tools::file_path_sans_ext(nm)),
+        ".", tools::file_ext(nm)
+      )
+      target <- file.path(fonts_home, nm)
+    }
+    sass:::download_file(url, target)
     css <<- sub(url, file.path("fonts", nm), css, fixed = TRUE)
   }, urls, basenames)
   writeLines(css, css_file)
+  NULL
 }
 
-themes <- list.dirs(
-  "inst/lib/bsw/dist",
+
+themes5 <- list.dirs(
+  "inst/lib/bsw5/dist",
+  recursive = FALSE,
+  full.names = TRUE
+)
+themes4 <- list.dirs(
+  "inst/lib/bsw4/dist",
   recursive = FALSE,
   full.names = TRUE
 )
@@ -48,5 +66,6 @@ themes3 <- list.dirs(
   full.names = TRUE
 )
 
-lapply(themes, download_and_copy_fonts)
+lapply(themes5, download_and_copy_fonts)
+lapply(themes4, download_and_copy_fonts)
 lapply(themes3, download_and_copy_fonts)
